import quandl
import time
from .google import *

# attribute is for HK stocks only, can be either 'unadj_close' or 'simple_ret'


def quandlPrice(tics=[], mkt='US', year=3,
                begdate='', enddate='', APIkey=''):
    quandl.ApiConfig.api_key = APIkey
    if (enddate == ''):
        enddate = datetime.now()
        begdate = enddate - relativedelta(years=year)
    else:
        enddate = datetime.strptime(str(enddate), '%Y%m%d')
        begdate = datetime.strptime(str(begdate), '%Y%m%d')

    enddate = enddate.strftime("%Y-%m-%d")
    begdate = begdate.strftime("%Y-%m-%d")

    count = 0
    while True:
        try:
            count += 1
            if (mkt == 'US' or mkt == 'us'):
                print("Getting data from quandl:WIKI/PRICES")
                df = quandl_us(tics, begdate, enddate)
            elif(mkt == 'HK' or mkt == 'hk'):
                print("Getting data from quandl:HKEX")
                dfa = quandl.get(['HKEX/%s.1' % (x) for x in tics],
                                 start_date=begdate, end_date=enddate)
                dfa = dfa / dfa.shift(1) - 1  # convert to simple return
                dfa = dfa.fillna(0)
                dfb = quandl.get(['HKEX/%s.12' % (x) for x in tics],
                                 start_date=begdate, end_date=enddate)
                df = pd.concat([dfa, dfb], axis=1)
                df = df.iloc[1:]
                df.columns = [['%s.HK_ret' % x for x in tics] +
                              ['%s.HK_boardlot' % x for x in tics]]

            else:
                print('Market must be US or HK.')
                return
            break
        except Exception:
            if count >= 5:
                print('Network Err')
                return
            else:
                time.sleep(1 * count)
                continue
    print('quandlPrice successful')
    return df


def quandl_us(tics, begdate, enddate):
    data = []
    df = pd.DataFrame()
    tasks = [x * 50 for x in range(len(tics) / 50 + 1)] + [len(tics)]
    for i in range(1, len(tasks)):
        print('Getting chunk %s ' % i)
        d = quandl.get_table('WIKI/PRICES',
                             qopts={'columns': [
                                    'ticker', 'date', 'adj_close']},
                             ticker=tics[tasks[i - 1]: tasks[i]],
                             date={'gte': begdate,
                                   'lte': enddate},
                             paginate=True)
        gpt = d.groupby('ticker')
        for ticker, df in gpt:
            df = df.set_index('date').drop('ticker', axis=1)
            df.columns = [ticker]
            data.append(df)
    if data:
        df = pd.concat(data, axis=1)
    return df


def HKPrice_formatter(df):
    cols = df.columns.to_series()
    tickers = [x[:-4] for x in cols][:(len(cols)) / 2]
    dfa = df[df.columns[cols.str.contains('_ret')]]
    dfa.columns = tickers
    return dfa


def HKBoardLot_formatter(df):
    cols = df.columns.to_series()
    tickers = [x[:-4] for x in cols][:(len(cols)) / 2]
    dfb = df[df.columns[cols.str.contains('_boardlot')]]
    dfb.columns = tickers
    dfb = dfb.fillna(method='ffill').iloc[-1]
    dfb = dfb.fillna(-999).astype(int)
    dfb = pd.DataFrame(dfb)
    dfb.columns = ['Board_Lot']
    return dfb


if __name__ == '__main__':
    from .datareader import *
    tics1 = getCons('SP500').iloc[:, 0].values.tolist()
    tics2 = getCons('HSI').iloc[:, 0].values.tolist()

    df1 = quandlPrice(tics1, mkt='US')
    df2 = quandlPrice(tics2, mkt='HK')



import requests
import os
import time
from bs4 import BeautifulSoup


def web_crawler(link):
    count = 0
    while True:
        try:
            # Pretend browser
            header = {
                "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.75 Safari/537.36",
                "X-Requested-With": "XMLHttpRequest"
            }
            r = requests.get(link, headers=header)
            count += 1
            break
        except Exception:
            if count >= 5:
                print('Network Err')
                break
            else:
                time.sleep(1 * count)
                continue

    return (r.text)


def bs_table_extractor(link, classname):
    """
    return all tables from a website with the classname
    """
    text = web_crawler(link)
    soup = BeautifulSoup(text, "html.parser")
    tables = soup.find_all("table", {"class": classname})
    data = []
    for table in tables:
        temp = []
        for row in table.findAll("tr"):
            cols = row.find_all('td')
            cols = [ele.text.strip() for ele in cols]
            temp.append(cols)
        data.append(temp)
    return data


def makepath(path):
    try:
        os.makedirs(path)
        print('Folder Created: %s' % path)
    except Exception:
        print('Folder Exist: %s' % path)
        pass
    return


def setup(event):
    global unpaused
    unpaused = event


def etfcom_extractor(tic):
    link = 'http://www.etf.com/' + tic
    text = web_crawler(link)
    soup = BeautifulSoup(text, "html.parser")
    content = {}
    if not (soup.title.text == 'Sorry! | ETF.com'):
        a = soup.find_all("div", class_="breadcrumb")[0]
        content['Segment'] = str(a.text[19:])

        # general information
        a = soup.find_all("div", class_="generalData")
        content['Fund Description'] = a[0].find('p').text.encode('utf-8')
        # Other data
        for n in range(3, 6):
            c = a[n]
            rows = c.find_all("div", class_="rowText")
            for row in rows:
                field = str(row.find('label').text)
                desc = str(row.find('span').text)
                content[field] = desc
                content['Ticker'] = tic
    else:
        print("%s Not found" % link)
    return content


def attempt(methodToRun, *args):
    count = 0
    while True:
        try:
            time.sleep(1 * count)
            count += 1
            result = methodToRun(*args)
            return result
        except Exception:
            if count >= 5:
                print('Maximum Retry Reached')
                return
            else:
                continue
    return



from datetime import datetime, timedelta
import pandas as pd
import pandas_datareader.data as web
from .utilities import bs_table_extractor, etfcom_extractor, setup
from dateutil.relativedelta import relativedelta
import warnings
import time
from multiprocessing import Pool, Event
import tushare as ts


warnings.filterwarnings('ignore')


def getCons(index):
    print('Getting %s constituents' % index)
    if(index == 'HSI'):
        base = "http://www.hsi.com.hk/HSI-Net/HSI-Net?"
        loc = "cmd=navigation&pageId=en.indexes.hsisis.hsi.constituents"
        tables = bs_table_extractor(base + loc, 'greygeneraltxt')[1:5]
        df = pd.concat([pd.DataFrame(x[2:]) for x in tables]).iloc[:, 1:3]
        df.columns = ['Symbol', 'Name']
        df.Symbol = df.Symbol.apply(lambda x: str(x).zfill(5))
        df = df.sort_values('Symbol')
        df = df.reset_index(drop=True)

    elif(index == 'SP500'):
        df = pd.read_html(
            "https://en.wikipedia.org/wiki/List_of_S%26P_500_companies")[0]
        df.columns = df.iloc[0, :]
        df = df.drop(0)
        df = df.drop('SEC filings', axis=1)

    elif(index == 'SP100'):
        df = pd.read_html(
            "https://en.wikipedia.org/wiki/S%26P_100")[2]
        df.columns = df.iloc[0, :]
        df = df.drop(0)

    elif(index == 'HS300'):
        df = ts.get_hs300s()

    elif(index == 'SZ50'):
        df = ts.get_sz50s()

    elif(index == 'ZZ500'):
        df = ts.get_zz500s()

    else:
        print('Index Error')
        return

    if isinstance(df, pd.DataFrame):
        print('getCons successful')

    return df

# Deprecated due to HKEX Website Update , Dec 2017
# def getBoardLot():
#     print('Connecting to hkex')
#     adr = 'https://www.hkex.com.hk/eng/market/sec_tradinfo/stockcode/'
#     security_type = {
#         'Equity Securities': 'eisdeqty.htm',
#         'Depositary Receipts (HDRs)': 'eisdhdr.htm',
#         'Equity Warrants': 'eisdew.htm',
#         'Investment Companies': 'eisdic.htm',
#         'Exchange Traded Funds (ETFs)': 'eisdetf.htm',
#         'Leveraged and Inverse Products (L&I Products)': 'liproducts.htm',
#         'Real Estate Investment Trusts (REITs)': 'eisdreit.htm',
#         'Other Unit Trusts/Mutual Funds': 'eisdtrus.htm',
#         'Derivative Warrants (DWs)': 'eisdwarr.htm',
#         'Callable Bull/Bear Contracts (CBBCs)': 'eisdcbbc.htm',
#         'Trading Only Securities': 'eisdnadq.htm',
#         'GEM Equity Securities': 'eisdgems.htm',
#         'GEM Equity Warrants': 'eisdgemw.htm'
#     }
#     df_all = []
#     for item in sorted(security_type):
#         print('Getting %s' % item)
#         link = adr + security_type[item]
#         table = bs_table_extractor(link, 'table_grey_border')
#         df = pd.DataFrame(table[0][1:]).iloc[:, :-4]
#         df.columns = [x.replace('\r\n\t\t\t\t\t                 ', '')
#                       for x in table[0][0][:-1]]
#         df['BOARD LOT'] = df['BOARD LOT'].apply(
#             lambda x: int(x.replace(',', '')))
#         df_all.append(df)
#     df = pd.concat(df_all)
#     df = df.sort_values('STOCK CODE')
#     df = df.reset_index(drop=True)
#     print('getBoardLot successful')
#     return df


def getLIBOR(year=3, begdate='', enddate='',
             data=['ONT', '1WK', '1MT', '3MT', '6MT', '12M'], currency=['USD']
             ):
    if (enddate == ''):
        enddate = datetime.now()
        begdate = enddate - relativedelta(years=year)
    else:
        enddate = datetime.strptime(str(enddate), '%Y%m%d')
        begdate = datetime.strptime(str(begdate), '%Y%m%d')
    for curr in currency:
        print('Getting %s LIBOR ' % curr)
        series_name = []
        for item in data:
            series_name.append("%s%sD156N" % (curr, item))
        df = web.DataReader(series_name, 'fred', begdate, enddate)
        df.columns = data

        # fill to most recent day
        newlist = []
        last_line = df.ix[-1].tolist()
        begin = df.index[-1].to_pydatetime().date()
        end = datetime.now().date()
        i = 1
        while True:
            new_timestamp = begin + timedelta(days=i)
            newlist.append([new_timestamp] + last_line)
            if(new_timestamp == end):
                break
            i += 1
        df1 = pd.DataFrame(newlist)
        df1 = df1.set_index(0)
        df1.columns = df.columns
        df = df.append(df1)
        df.index = pd.DatetimeIndex(df.index).normalize()
    print('getLIBOR successful')
    return df


def getIndex_yahoo(index, year=3, begdate='', enddate=''):
    if (enddate == ''):
        enddate = datetime.now()
        begdate = enddate - relativedelta(years=year)
    else:
        enddate = datetime.strptime(str(enddate), '%Y%m%d')
        begdate = datetime.strptime(str(begdate), '%Y%m%d')
    if (index == 'SP500'):
        series_name = '^GSPC'
    elif (index == 'HSI'):
        series_name = '^HSI'
    else:
        print('Err: Unsupported Index')
        return
    df = web.DataReader(series_name, 'yahoo', begdate, enddate)
    df = df[['Adj Close']]
    df.columns = [index]
    return df


def getETF(mkt=''):
    if (mkt == 'US' or mkt == 'us'):
        exchanges = {'Chicago Stock Exchange (CHX)': 'chx',
                     'NASDAQ OMX BX (BEX)': 'bex',
                     'NYSE Arca (ARCA)': 'arca',
                     'NYSE MKT (NYSE AMEX)': 'amex'}
        etf_link = 'https://www.interactivebrokers.com.hk/en/index.php?f=567&exch='
        etf_list = []
        for exch in sorted(exchanges):
            print('Getting ETF list of %s' % exch)
            link = etf_link + exchanges[exch]
            df_t = pd.read_html(link)[2]
            etf_list.append(df_t)

        full_list = pd.concat(etf_list).drop_duplicates().sort_values(
            'IB Symbol').reset_index(drop=True)
        columns = list(full_list.columns)
        columns[1] = 'Description'
        full_list.columns = columns

    elif (mkt == 'HK' or mkt == 'hk'):
        print('Getting ETF list from IB')
        adr_ib = 'https://www.interactivebrokers.com.hk/en/index.php?f=567&exch=sehk'
        df_ib = pd.read_html(adr_ib)[2]
        seclist_ib = df_ib['Symbol'].values.tolist()

        # Deprecated due to HKEX Website Update , Dec 2017
        # print('Getting ETF list from HKEX')
        # adr_hkex = 'https://www.hkex.com.hk/eng/market/sec_tradinfo/stockcode/eisdetf.htm'
        # a = bs_table_extractor(adr_hkex, 'table_grey_border')
        # seclist_hkex = pd.DataFrame(a[0][1:])[0].values.tolist()

        # hk_etf_list = []
        # for sec in seclist_hkex:
        #     idsec = int(sec)
        #     if idsec in seclist_ib:
        #         hk_etf_list.append(idsec)

        # full_list = df_ib[df_ib.Symbol.apply(lambda x: x in hk_etf_list)]
        # columns = list(full_list.columns)
        # columns[1] = 'Description'
        # full_list.columns = columns

        full_list = df_ib
        columns = list(full_list.columns)
        columns[1] = 'Description'
        full_list.columns = columns
    else:
        print('Market must be US or HK')
        return
    print('getETF successful')
    return full_list


def getETFinfo(mkt='', processes=10):
    if (mkt == 'US' or mkt == 'us'):
        print('Getting ETF info from ETF.com')
        etf_list = getETF(mkt='US')
        tics = map(str, etf_list.Symbol.values.tolist())

        event = Event()
        p = Pool(processes, setup, (event,))
        rs = p.map_async(etfcom_extractor, tics, chunksize=1)
        event.set()
        while not rs.ready():
            time.sleep(60)
            event.clear()  # pause after five seconds
            print("PAUSED, {} left".format(rs._number_left))
            time.sleep(5)
            event.set()
            print("RESUMED")
        ac = rs.get()
        info = []
        for content in ac:
            if content:
                info.append(content)
        info = pd.DataFrame(info).drop('', axis=1)
        info = info.set_index('Ticker')
    elif (mkt == 'HK' or mkt == 'hk'):
        # print('Getting ETF info from HKEX')
        # link = 'https://www.hkex.com.hk/eng/etfrc/ETFTA/ETFTradingArrangement.htm'
        # classname = 'table_grey_border'
        # data = bs_table_extractor(link, classname)

        # res = []
        # for a in data:
        #     df = pd.DataFrame(a[1:], columns=a[0] + ['unimp'])
        #     df = df.iloc[:, 1:15]
        #     res.append(df)
        #     info = pd.concat(res)
        print ('HKEX Website Change, Pending Implementation')
        return
    else:
        print('Market must be US or HK')
        return
    print('getETFinfo successful')
    return info


if __name__ == '__main__':
    df1 = getCons('SP500')
    df2 = getCons('SP100')
    df3 = getCons('HSI')
    # boardlot = getBoardLot()
    Libor = getLIBOR(year=3)
    etfhk = getETF('HK')
    etfus = getETF('US')
    etf_info_hk = getETFinfo('HK')
    etf_info_US = getETFinfo('US')



from tushare import get_h_data
from datetime import datetime
from dateutil.relativedelta import relativedelta
import pandas as pd
from utilities import attempt


def tusharePrice(tics=[], year=1, begdate='', enddate='',
                 autype='', index=False):

    if (enddate == ''):
        enddate = datetime.now()
        begdate = enddate - relativedelta(years=year)
    else:
        enddate = datetime.strptime(str(enddate), '%Y%m%d')
        begdate = datetime.strptime(str(begdate), '%Y%m%d')

    enddate = enddate.strftime("%Y-%m-%d")
    begdate = begdate.strftime("%Y-%m-%d")

    rs = {}
    skipped = []
    for tic in tics:
        df = attempt(get_h_data, tic, begdate, enddate, autype, index, 5, 5)
        if not df.empty:
            rs[tic] = df
        else:
            skipped.append(tic)

    if skipped:
        print('Retry %s' % skipped)
        for tic in skipped:
            df = attempt(get_h_data, tic, begdate,
                         enddate, autype, index, 5, 5)
            if not df.empty:
                rs[tic] = df
            else:
                print('Warning: %s is missing. skipped' % tic)

    wp = pd.Panel.from_dict(rs, orient='minor')
    return wp



from setuptools import setup, find_packages

setup(name='findar',
      version='0.37',
      description='Financial Datareader (findar)',
      long_description='Financial Datareader provides data solutions to quantitative trading and \
      backtesting.Currently provides up-to-date index constituent lists, full ETF lists \
      trading boardlots, fundamental informations, Libor rates etc, for both US and HK market.',
      url='https://github.com/wwengm/findar',
      author='wwengm',
      license='MIT',
      zip_safe=False,
      keywords='financial data',
      packages=find_packages(),
      install_requires=['pandas', 'pandas_datareader', 'beautifulsoup4', 'requests', 'quandl', 'tushare'],)



from .utilities import web_crawler
from datetime import datetime
import pandas as pd
from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta
import pandas_datareader.data as web
import warnings


class google(object):
    """
    Get daily stock prices from google
    For most US stocks, through API
    If failed, download through web_crawling
    """
    warnings.filterwarnings('ignore')
    base = 'https://www.google.com/finance/historical?'

    def __init__(self, tics=[], year=3, begdate='', enddate=''):
        self.data = {}
        self.tics = tics
        self.remain = tics
        if (enddate == ''):
            self.enddate = datetime.now()
            self.begdate = self.enddate - relativedelta(years=year)
            self.days = year * 365
        else:
            self.enddate = datetime.strptime(str(enddate), '%Y%m%d')
            self.begdate = datetime.strptime(str(begdate), '%Y%m%d')
            self.days = int(abs((self.enddate - self.begdate).days))

    def url_generator(self, symbol):
        n = int(self.days / 200) + 1
        url_list = []
        stock = 'q=%s' % symbol
        start = '&startdate=%s' % self.date_formatter(self.begdate)
        end = '&enddate=%s' % self.date_formatter(self.enddate)
        num = '&num=200'
        for i in range(n):
            start_row = '&start=%s' % (i * 200)
            url = self.base + stock + start + end + start_row + num
            url_list.append(url)
        return url_list

    def core(self):
        for symbol in self.remain:
            urls = self.url_generator(symbol)
            raw = []
            for url in urls:
                page = web_crawler(url)
                if ('Google automatically detects requests' in page):
                    raise Exception('GoogleTrafficBlock:Retry Later')

                soup = BeautifulSoup(page, "lxml")
                table = soup.select_one("table.gf-table.historical_price")
                if (table):
                    all_rows = table.find_all("tr")
                    raw += [x.text.split('\n') for x in all_rows[1:]]
                else:
                    break

            if (raw):
                try:
                    self.data[symbol] = self.result_formatter(raw)
                except:
                    print raw
                    print urls
        return

    def run(self):
        """
        Main Function: First obtain data from API
        If failed then obain by web crawling
        """
        try:
            print('Connecting Google API')
            wp = web.DataReader(self.tics, 'google',
                                self.begdate, self.enddate)
            exist = wp.minor_axis.values.tolist()
            for tic in exist:
                df = wp.minor_xs(tic)
                if not (df.dropna().empty):
                    df = df.applymap(lambda x: str(x).replace(',', ''))
                    self.data[tic] = df
            self.remain = [x for x in self.tics if x not in self.data]
            if (self.remain):
                print('%s stocks remains, searching Google_Web' %
                      len(self.remain))
        except Exception:
            pass
        self.core()
        self.remain = [x for x in self.tics if x not in self.data]
        wp = pd.Panel.from_dict(self.data, orient='minor')
        if (self.remain):
            print('%s not found on google Finance. Please check tickers' %
                  self.remain)
        return wp

    """
    Some Helper Functions
    """

    def date_formatter(self, datetime_object):
        month = datetime_object.strftime("%B")
        day = str(datetime_object.day).zfill(2)
        year = datetime_object.year
        fmt_date = '%s+%s%%2C+%s' % (month, day, year)
        return (fmt_date)

    def result_formatter(self, raw):
        df = pd.DataFrame(raw).drop_duplicates().iloc[:, 1:7]
        df.columns = ['Timestamp', 'Open', 'High',
                      'Low', 'Close', 'Volume']
        df.Timestamp = pd.to_datetime(df.Timestamp)
        df = df.sort_values('Timestamp')
        df = df.set_index('Timestamp')
        df = df.applymap(lambda x: str(x).replace(',', ''))
        return df


def googlePrice(tics=[], year=3, begdate='', enddate=''):
    a = google(tics, year, begdate, enddate)
    wp = a.run()
    return wp


if __name__ == '__main__':
    a = google(tics=['AAPL', 'HKG:0005', 'a_fake_ticker'], year=3)
    wpa = a.run()

    b = google(tics=['AAPL', 'HKG:0005', 'a_fake_ticker'],
               begdate=20160101, enddate=20170501)
    wpb = b.run()
